/*
 * The Samsung X205L/210L sports camcorders generate semi-proprietary 
 * .avi files which can not be properly read under OS X.  However,
 * by making a few changes to the file, you can play the .avi files
 * using VLC.
 *
 * To compile:
 * gcc -o sedg2divx -O2 sedg2divx.c
 *
 * To convert SEDG AVI files to something VLC can play:
 * sedg2divx <file>
 */

/*
 *  Copyright (c) 2007 Aaron Turner.  aturner at synfin dot net
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the names of the copyright owners nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 *  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *  GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 *  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 *  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>


/* search/replace and byte offsets (starting at zero) */
#define SEARCH  "SEDG"
#define REPLACE "DIVX"
int offsets[2] = {112, 188};

/* must be at least 4 bytes larger then the largest offset */
size_t buflen = 1024; 

void usage(const char *);

int
main(int argc, char *argv[])
{
    int fd, i;
    char *buffer, tempbuf[5];
    int offsets_cnt = sizeof(offsets) / sizeof(int);

    /* we require one option: the file name */
    if (argc < 2)
        usage(argv[0]);

    /* open the file */
    if ((fd = open(argv[1], O_RDWR, 0)) < 0) {
        fprintf(stderr, "Unable to open file: %s (%s)\n", 
                argv[1], strerror(errno));
        exit(-1);
    }

    /* mmap the file */
    if ((buffer = mmap(NULL, buflen, PROT_READ|PROT_WRITE, MAP_FILE|MAP_SHARED, fd, 0)) == NULL) {
        fprintf(stderr, "Unable to mmap file: %s\n",
                argv[1], strerror(errno));
        exit(-1);
    }
  
    /* make our mods */
    for (i = 0; i < offsets_cnt; i++) {
        memcpy(tempbuf, &buffer[offsets[i]], strlen(SEARCH));
        tempbuf[4] = '\0';
        printf("Looking at offset %d: %s\n", offsets[i], tempbuf);
        if (strncmp(&buffer[offsets[i]], SEARCH, strlen(SEARCH)) == 0) {
            printf("Replacing token %d/%d at offset %d\n", i+1, offsets_cnt, offsets[i]);
            memcpy(&buffer[offsets[i]], REPLACE, strlen(REPLACE));
        }
    }

    /* sync & close */
    if (msync(buffer, buflen,  MS_SYNC) < 0) {
        fprintf(stderr, "Unable to sync chnages back to file: %s\n",
                strerror(errno));
        exit(-1);
    }
    munmap(buffer, buflen);
    close(fd);
    exit(0);
}

void
usage(const char *cmd)
{
    printf("Samsung SEDG to DIVX converter v0.1\n");
    printf("Copyright 2007 Aaron Turner <aturner at synfin dot net>\n");
    printf("%s <file>\n", cmd);
    exit(0);
}
